@@ -1,22 +1,26 @@
 package App::RPi::EnvUI;
 
 use App::RPi::EnvUI::API;
+use App::RPi::EnvUI::Auth;
 use Data::Dumper;
 use Dancer2;
 use Dancer2::Plugin::Auth::Extensible;
+use Dancer2::Core::Request;
 #use Dancer2::Session::JSON;
 use Mock::Sub no_warnings => 1;
+use POSIX qw(tzset);
 
-our $VERSION = '0.25';
+our $VERSION = '0.26';
 
 my $db = App::RPi::EnvUI::DB->new;
 my $api = App::RPi::EnvUI::API->new(db => $db);
 
+$ENV{TZ} = $api->_config_core('time_zone');
+tzset();
+
 my $log = $api->log()->child('webapp');
 
-$api->_reset();
 $api->_config_light();
-$api->env($api->read_sensor);
 
 #
 # fetch routes
@@ -25,20 +29,56 @@ $api->env($api->read_sensor);
 get '/' => sub {
         my $log = $log->child('/');
         $log->_7("entered");
+
         # return template 'main';
-        return template 'test';
+        return template 'test', {requester => request->address};
+        # return template 'switch';
+        # return template 'switch2';
+        # return template 'menu';
+        # return template 'drag';
+        # return template 'flip';
+        # return template 'graph';
+        # return template 'graph_live';
+    };
+
+# fetch graph code
+
+get '/graph_data' => sub {
+        return to_json $api->graph_data;
     };
 
 post '/login' => sub {
-        session user => params->{username};
-        session pass => $api->user(params->{username})->{pass};
-        redirect '/';
+        my $user = params->{username};
+        my $pass = params->{password};
+
+        my ($success, $realm) = authenticate_user($user, $pass);
+
+        if ($success){
+            session logged_in_user => $user;
+            session logged_in_user_realm => $realm;
+            redirect '/';
+        }
     };
 
 any '/logout' => sub {
         app->destroy_session;
     };
 
+get '/logged_in' => sub {
+        if (session 'logged_in_user'){
+            return to_json { status => 1 };
+        }
+        return to_json {status => 0};
+    };
+
+get '/time' => sub {
+        return join ':', (localtime)[2, 1];
+    };
+
+get '/stats' => sub {
+        return template 'stats';
+    };
+
 get '/light' => sub {
         my $log = $log->child('/light');
         $log->_7("entered");
@@ -98,6 +138,13 @@ get '/fetch_env' => sub {
 #
 
 get '/set_aux/:aux/:state' => sub {
+
+        if ((request->address ne '127.0.0.1' && ! session 'logged_in_user') || $ENV{UNIT_TEST}){
+            return to_json {
+                    error => 'unauthorized request. You must be logged in'
+            };
+        }
+
         my $aux_id = params->{aux};
         my $state = $api->_bool(params->{state});
 
