@@ -11,7 +11,7 @@ use Logging::Simple;
 use Mock::Sub no_warnings => 1;
 use RPi::WiringPi::Constant qw(:all);
 
-our $VERSION = '0.25';
+our $VERSION = '0.26';
 
 # mocked sub handles for when we're in testing mode
 
@@ -44,11 +44,7 @@ sub new {
     return $self;
 }
 sub action_humidity {
-    #FIXME: this, and action_temp() should retrieve their own aux id
-    # from the db directly, instead of having it passed in?
-
-    my $self = shift;
-    my ($aux_id, $humidity) = @_;
+    my ($self, $aux_id, $humidity) = @_;
 
     my $log = $log->child('action_humidity');
     $log->_5("aux: $aux_id, humidity: $humidity");
@@ -70,55 +66,11 @@ sub action_humidity {
             $self->aux_state($aux_id, LOW);
             $self->aux_time($aux_id, 0);
         }
-    }
-}
-sub action_light {
-    my ($self) = @_;
-
-    my $log = $log->child('action_light');
-
-    my $now = DateTime->now(
-        time_zone => $self->_config_core('time_zone')
-    );
-
-    my ($on_hour, $on_min) = split /:/, $self->_config_light('on_at');
-
-    $log->_7("on_hour: $on_hour, on_min: $on_min");
-
-    my $timer = $now->clone;
-    $timer->set(hour => $on_hour);
-    $timer->set(minute => $on_min);
-
-    my $hour_same = $now->hour == $timer->hour;
-    my $min_geq = $now->minute >= $timer->minute;
-
-    my $on_since = $self->_config_light('on_since');
-
-    if (! $on_since  && $hour_same && $min_geq){
-        $self->db()->update('light', 'value', time(), 'id', 'on_since');
-        $self->aux_state($self->_config_control('light_aux'), ON);
-        pin_mode($self->_config_control('light_aux'),  OUTPUT);
-        write_pin($self->aux_pin($self->_config_control('light_aux')), HIGH);
-    }
-
-    if ($on_since) {
-        my $on_hours = $self->_config_light( 'on_hours' );
-        my $on_secs = $on_hours * 60 * 60;
-
-        my $t = time();
-        my $diff = $t - $on_since;
-
-        if ($diff > $on_secs) {
-            $self->db()->update( 'light', 'value', 0, 'id', 'on_since' );
-            $self->aux_state( $self->_config_control( 'light_aux' ), OFF );
-            pin_mode($self->_config_control('light_aux'),  OUTPUT);
-            write_pin($self->aux_pin($self->_config_control('light_aux')), LOW);
-        }
+        $self->switch($aux_id);
     }
 }
 sub action_temp {
-    my $self = shift;
-    my ($aux_id, $temp) = @_;
+    my ($self, $aux_id, $temp) = @_;
 
     my $log = $log->child('action_temp');
 
@@ -138,31 +90,45 @@ sub action_temp {
             $self->aux_state($aux_id, LOW);
             $self->aux_time($aux_id, 0);
         }
+        $self->switch($aux_id);
     }
 }
 sub action_light {
-    my ($self) = @_;
+    my ($self, $dt) = @_;
 
     my $log = $log->child('action_light');
+    
+    my $now;
 
-    my $now = DateTime->now(
-        time_zone => $self->_config_core('time_zone')
-    );
+    if (defined $dt){
+        $now = $dt;
+    }
+    else {
+        $now = DateTime->now(time_zone => $self->_config_core('time_zone'));
+    }
 
-    my ($on_hour, $on_min) = split /:/, $self->_config_light('on_at');
+    if ($self->_config_light('on_hours') == 0){
+        $self->aux_state($self->_config_control('light_aux'), OFF);
+        write_pin($self->aux_pin($self->_config_control('light_aux')), LOW);
+        return;
+    }
+    if ($self->_config_light('on_hours') == 24){
+        if(! $self->aux_state($self->_config_control('light_aux'))){
+            $self->db()->update('light', 'value', time(), 'id', 'on_since');
+            $self->aux_state($self->_config_control('light_aux'), ON);
+            pin_mode($self->_config_control('light_aux'),  OUTPUT);
+            write_pin($self->aux_pin($self->_config_control('light_aux')), HIGH);
+        }
+        return;
+    }
 
-    $log->_7("on_hour: $on_hour, on_min: $on_min");
+    my $on_since  = $self->_config_light('on_since');
+    my $light_on  = $self->light_on;
 
-    my $timer = $now->clone;
-    $timer->set(hour => $on_hour);
-    $timer->set(minute => $on_min);
+    # print "now: " . $now->ymd . " " . $now->hms . "\n";
+    # print "on: " . $light_on->ymd . " " . $light_on->hms . "\n";
 
-    my $hour_same = $now->hour == $timer->hour;
-    my $min_geq = $now->minute >= $timer->minute;
-
-    my $on_since = $self->_config_light('on_since');
-
-    if (! $on_since  && $hour_same && $min_geq){
+    if (! $on_since  && $now > $light_on){
         $self->db()->update('light', 'value', time(), 'id', 'on_since');
         $self->aux_state($self->_config_control('light_aux'), ON);
         pin_mode($self->_config_control('light_aux'),  OUTPUT);
@@ -170,16 +136,12 @@ sub action_light {
     }
 
     if ($on_since) {
-        my $on_hours = $self->_config_light( 'on_hours' );
-        my $on_secs = $on_hours * 60 * 60;
-
-        my $t = time();
-        my $diff = $t - $on_since;
-
-        if ($diff > $on_secs) {
-            $self->db()->update( 'light', 'value', 0, 'id', 'on_since' );
-            $self->aux_state( $self->_config_control( 'light_aux' ), OFF );
-            pin_mode($self->_config_control('light_aux'),  OUTPUT);
+        my $light_off = $self->light_off;
+        # print "off: " . $light_off->ymd . " " . $light_off->hms . "\n";
+        if ($now > $light_off) {
+            $self->db()->update('light', 'value', 0, 'id', 'on_since');
+            $self->aux_state( $self->_config_control('light_aux'), OFF);
+            pin_mode($self->_config_control('light_aux'), OUTPUT);
             write_pin($self->aux_pin($self->_config_control('light_aux')), LOW);
         }
     }
@@ -228,7 +190,6 @@ sub aux_override {
 sub aux_pin {
     my $self = shift;
     # returns the auxillary's GPIO pin number
-
     my ($aux_id, $pin) = @_;
 
     if ($aux_id !~ /^aux/){
@@ -303,6 +264,44 @@ sub env {
     return {temp => -1, humidity => -1} if ! defined $ret;
     return $self->db()->env;
 }
+sub graph_data {
+    my ($self) = @_;
+
+    my $graph_data = $self->db()->graph_data;
+
+    my $check = 1;
+    my $count = 0;
+    my %data;
+
+    my $need = 5760 - @$graph_data; # approx 4 per min, for 24 hours (4*60*24)
+
+    for (@$graph_data) {
+
+        # we need to pad out to get to 24 hours worth of valid data
+
+        if ($need){
+            my $last_t = $_->[2];
+            my $last_h = $_->[3];
+            while($need){
+                push @{ $data{temp} }, [ $count, $last_t ];
+                push @{ $data{humidity} }, [ $count, $last_h ];
+                $need--;
+                $count++;
+            }
+        }
+
+
+        # every 4 entries; typically we have 4 polls per minute
+        if ($check % 4 == 1) {
+            push @{ $data{temp} }, [ $count, $_->[2] ];
+            push @{ $data{humidity} }, [ $count, $_->[3] ];
+            $count++;
+        }
+        $check++;
+    }
+
+    return \%data;
+}
 sub humidity {
     my $self = shift;
     return $self->env()->{humidity};
@@ -414,6 +413,7 @@ sub user {
 
     return $self->db()->user($un);
 }
+
 # public configuration getters
 
 sub env_humidity_aux {
@@ -424,6 +424,40 @@ sub env_temp_aux {
     my $self = shift;
     return $self->_config_control('temp_aux');
 }
+sub light_on {
+    my ($self) = @_;
+
+    my ($on_hour, $on_min) = split /:/, $self->_config_light('on_at');
+
+    my $now = DateTime->now(
+        time_zone => $self->db()->config_core('time_zone')
+    );
+
+    my $on
+      = $now->clone()->set_hour($on_hour)->set_minute($on_min)->set_second(0);
+
+    return $on;
+}
+sub light_off {
+    my ($self, $dt) = @_;
+
+    my $on_hours = $self->_config_light('on_hours');
+    my $on_since = $self->_config_light('on_since');
+
+    if (defined $dt){
+        return $dt->clone()->add(hours => $on_hours);
+    }
+    else {
+        $dt = DateTime->from_epoch(
+            time_zone => $self->_config_core('time_zone'),
+            epoch => $on_since
+        );
+
+        $dt->add(hours => $on_hours);
+    }
+
+    return $dt;
+}
 
 # public instance variable methods
 
@@ -531,25 +565,21 @@ sub _config_light {
     my $self = shift;
     my $want = shift;
 
-    my $light = $self->db()->config_light;
-
     my %conf;
 
-    for (keys %$light) {
+    my $light = $self->db()->config_light;
+
+    for (keys %$light){
+        if ($_ eq 'on_hours'){
+            my $on_hrs = $light->{on_hours}->{value};
+            if ($on_hrs !~ /^\d+$/ || $on_hrs < 0 || $on_hrs > 24){
+                confess "\n\non_hours config file directive must be between ".
+                        "0 and 24\n\n"
+            }
+        }
         $conf{$_} = $light->{$_}{value};
     }
 
-    my ($on_hour, $on_min) = split /:/, $conf{on_at};
-
-    my $now = DateTime->now(time_zone => $self->db()->config_core('time_zone'));
-    my $light_on = $now->clone;
-
-    $light_on->set_hour($on_hour);
-    $light_on->set_minute($on_min);
-
-    my $dur = $now->subtract_datetime($light_on);
-    $conf{on_in} = $dur->hours . ' hrs, ' . $dur->minutes . ' mins';
-
     if (defined $want){
         return $conf{$want};
     }
@@ -686,6 +716,12 @@ sub _log {
 sub _parse_config {
     my ($self, $config) = @_;
 
+
+    $self->db()->begin;
+
+    $self->_reset;
+    $self->_reset_light;
+
     $config = $self->config if ! defined $config;
 
     if (! -e $config){
@@ -702,21 +738,37 @@ sub _parse_config {
 
     # auxillary channels
 
-    for (1..8){
-        my $aux_id = "aux$_";
-        my $pin = $conf->{$aux_id}{pin};
-        $self->aux_pin($aux_id, $pin);
+    {
+        my $db_struct = [
+            'aux',
+            'pin',
+            'id'
+        ];
+        my @data;
+
+        for (1 .. 8) {
+            my $aux_id = "aux$_";
+            my $pin = $conf->{$aux_id}{pin};
+            push @data, [$pin, $aux_id];
+            #$self->aux_pin( $aux_id, $pin );
+        }
+
+        $self->db()->update_bulk(@$db_struct, \@data);
     }
 
     for my $conf_section (qw(control core light water)){
+        my $db_struct = [
+            $conf_section,
+            'value',
+            'id'
+        ];
+        my @data;
+
         for my $directive (keys %{ $conf->{$conf_section} }){
-            $self->db()->update(
-                $conf_section,
-                'value',
+            push @data, [
                 $conf->{$conf_section}{$directive},
-                'id',
                 $directive
-            );
+            ];
 
             # populate some internal variables from the 'core'
             # config section
@@ -726,18 +778,30 @@ sub _parse_config {
                 $self->{$directive} = $conf->{$conf_section}{$directive};
             }
         }
+
+        $self->db()->update_bulk(@$db_struct, \@data);
     }
+
+    $self->db()->commit;
 }
 sub _reset {
     my $self = shift;
     # reset dynamic db attributes
 
-    for (1..8){
-        my $aux_id = "aux$_";
-        $self->aux_time($aux_id, 0);
-        $self->aux_state($aux_id, 0);
-        $self->aux_override($aux_id, 0);
-    }
+    $self->db()->update_bulk_all(
+        'aux', 'state', [0]
+    );
+    $self->db()->update_bulk_all(
+        'aux', 'override', [0]
+    );
+    $self->db()->update_bulk_all(
+        'aux', 'on_time', [0]
+    );
+
+    #$self->db()->commit;
+}
+sub _reset_light {
+    $_[0]->db()->update('light', 'value', 0, 'id', 'on_since');
 }
 sub _ui_test_mode {
     return -e 't/testing.lck';
@@ -834,13 +898,19 @@ default, this will be C<aux2>.
 Mandatory: Integer. The integer value of the current humidity (typically
 supplied by the C<RPi::DHT11> hygrometer sensor.
 
-=head2 action_light
+=head2 action_light($dt)
 
 Performs the time calculations on the configured light on/off event settings,
 and turns the GPIO pin associated with the light auxillary channel on and off as
 required.
 
-Takes no parameters.
+Parameters (only used for testing):
+
+    $dt
+
+Optional (use for testing only!), L<DateTime> object. This object will replace
+a call to C<now()>, so that when we're testing, we can simulate now as being in
+the future to test on/off triggers properly.
 
 =head2 action_temp($aux_id, $temperature)
 
@@ -1060,6 +1130,15 @@ processes, performing actions outside of the main thread.
 
 Takes no parameters, has no return.
 
+=head2 graph_data
+
+Returns a hash reference with keys C<temp> and C<humidity>, where the values of
+each are an array reference of array references with the inner arefs containing
+the element number and the temp/humidity value.
+
+It attempts to fetch data for 24 hours, sampling approximately every minute. If
+no data is found far enough back, the temp/humidity will be set to C<0>.
+
 =head2 humidity
 
 Returns as an integer, the current humidity level.
@@ -1068,6 +1147,25 @@ Returns as an integer, the current humidity level.
 
 Returns as an integer, the current temperature level.
 
+=head2 light_on
+
+Returns a L<DateTime> object with the C<HH:MM> set to the C<on_at> directive in
+the configuration file.
+
+=head2 light_off($dt)
+
+Returns a L<DateTime> object with the C<on_hours> config directive value added,
+which represents the time the light shall be turned off.
+
+Parameters:
+
+    $dt
+
+Optional, L<DateTime> object. Use only for testing. We'll use this as the on
+time C<DateTime> instead of one received from a call to C<light_on()>.
+
+Returns: C<DateTime> object.
+
 =head2 log
 
 Returns a pre-configured L<Logging::Simple> object, ready to be cloned with its
