@@ -1,11 +1,25 @@
 package App::RPi::EnvUI::DB;
 
-use Data::Dumper;
-use DateTime;
-use DBI;
-use RPi::WiringPi::Constant qw(:all);
+BEGIN {
+    if (! exists $INC{'DateTime.pm'}){
+        require DateTime;
+        DateTime->import;
+    }
+    if (! exists $INC{'DBI.pm'}){
+        require DBI;
+        DBI->import;
+    }
+    if (! exists $INC{'Data/Dumper.pm'}){
+        require Data::Dumper;
+        Data::Dumper->import;
+    }
+    if (! exists $INC{'RPi/WiringPi/Constant.pm'}){
+        require RPi::WiringPi::Constant;
+        DateTime->import(qw(:all));
+    }
+}
 
-our $VERSION = '0.25';
+our $VERSION = '0.26';
 
 sub new {
     my ($class, %args) = @_;
@@ -24,9 +38,22 @@ sub new {
         "dbi:SQLite:dbname=$db_file",
         "",
         "",
-        {RaiseError => $self->{db_err}}
+        {
+            #sqlite_use_immediate_transaction => 1,
+            RaiseError => $self->{db_err},
+            AutoCommit => 1
+        }
     ) or die $DBI::errstr;
 
+    # commonly used statement handles
+
+    $self->{graph_sth} = $self->{db}->prepare(
+        "select * from (
+            select * from stats order by id DESC limit 5760
+        ) sub
+            order by id asc;"
+    );
+
     return $self;
 }
 sub user {
@@ -40,8 +67,8 @@ sub user {
 
     my $res = $sth->fetchrow_hashref();
 
-    return ! ref $res eq 'HASH'
-        ? undef
+    return ref $res ne 'HASH'
+        ? {user => $user, pass => ''}
         : $res;
 }
 sub aux {
@@ -122,6 +149,11 @@ sub env {
     $sth->execute($id);
     return $sth->fetchrow_hashref;
 }
+sub graph_data {
+    my ($self) = @_;
+    $self->{graph_sth}->execute;
+    return $self->{graph_sth}->fetchall_arrayref;
+}
 sub insert_env {
     my ($self, $temp, $hum) = @_;
 
@@ -141,17 +173,43 @@ sub last_id {
 sub update {
     my ($self, $table, $col, $value, $where_col, $where_val) = @_;
 
-    if (! defined $where_col){
-        my $sth = $self->{db}->prepare("UPDATE $table SET $col=?");
-        $sth->execute($value);
+    if (! defined $where_col) {
+        my $sth = $self->{db}->prepare( "UPDATE $table SET $col=?" );
+        $sth->execute( $value );
     }
     else {
         my $sth = $self->{db}->prepare(
             "UPDATE $table SET $col=? WHERE $where_col=?"
         );
-        $sth->execute($value, $where_val);
+        $sth->execute( $value, $where_val );
     }
 }
+sub update_bulk {
+    my ($self, $table, $col, $where_col, $data) = @_;
+
+    my $sth = $self->{db}->prepare(
+        "UPDATE $table SET $col=? WHERE $where_col=?"
+    );
+
+    for (@$data){
+        $sth->execute(@$_);
+    }
+}
+sub update_bulk_all {
+    my ($self, $table, $col, $data) = @_;
+
+    my $sth = $self->{db}->prepare(
+        "UPDATE $table SET $col=?;"
+    );
+
+    $sth->execute(@$data);
+}
+sub begin {
+    $_[0]->{db}->begin_work;
+}
+sub commit {
+    $_[0]->{db}->commit;
+}
 
 true;
 __END__
@@ -220,6 +278,16 @@ top-level hash is a list of all the auxillary channel names, and each key has
 a value of another hash reference, containing the details of that specific
 aux channel. Takes no parameters.
 
+=head2 begin
+
+Start a transaction for bulk database writes. Used only in scopes that use
+C<update_bulk()> and C<update_bulk_all()>.
+
+=head2 commit
+
+Commit a transaction for bulk database writes. Used only in scopes that use
+C<update_bulk()> and C<update_bulk_all()>.
+
 =head2 config_control($want)
 
 Fetches and returns the value of a specific C<control> configuration variable.
@@ -274,6 +342,13 @@ Fetches and returns as a hash reference the last database entry of the C<stats>
 temperature/humidity update, along with a timestamp and row ID. Takes no
 parameters.
 
+=head2 graph_data
+
+Fetches and approximately 24 hours worth of rows starting from the last insert
+and going backwards.
+
+Returns an array reference that contain an array reference for each row found.
+
 =head2 insert_env($temp, $humidity)
 
 Inserts into the C<stats> database table a new row containing a row ID,
@@ -324,6 +399,25 @@ clause. This value is ignored if C<$where_col> is not specified.
 NOTE: If C<$where_col> is not sent in, we will operate on all rows in the
 specified table.
 
+=head2 update_bulk($table, $col, $where_col, $data)
+
+Bulk updates to the database consolidating statements for efficiency. Used
+primarily in the initialization phases.
+
+Before you set this call up, you must make a call to C<config()>, and after
+you're done, make a call to C<commit>.
+
+The parameters are the same as C<update()>, less:
+
+    $data
+
+Mandatory, array ref of array refs. Each inner array reference that contains
+the value to update to, and the where clause value.
+
+=head2 update_bulk_all($table, $col, $data)
+
+Same as C<update_bulk()> except operates on all table rows (there's no where
+clause).
 
 =head1 AUTHOR
 
