@@ -3,7 +3,7 @@ package App::RPi::EnvUI::Event;
 use Async::Event::Interval;
 use Data::Dumper;
 
-our $VERSION = '0.25';
+our $VERSION = '0.26';
 
 sub new {
     my ($class, %args) = @_;
@@ -51,7 +51,7 @@ sub env_action {
 
             $api->action_temp($t_aux, $api->temp);
             $api->action_humidity($h_aux, $api->humidity);
-            $api->action_light($api->_config_light)
+            $api->action_light
               if $api->_config_light('enable');
         }
     );
